@@ -4,7 +4,7 @@
  *	  utilities routines for the postgres GiST index access method.
  *
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -17,12 +17,16 @@
 
 #include "access/gist_private.h"
 #include "access/reloptions.h"
-#include "storage/freespace.h"
 #include "storage/indexfsm.h"
 #include "storage/lmgr.h"
-#include "storage/bufmgr.h"
-#include "utils/rel.h"
+#include "utils/builtins.h"
 
+/*
+ * static *S used for temporary storage (saves stack and palloc() call)
+ */
+
+static Datum attrS[INDEX_MAX_KEYS];
+static bool isnullS[INDEX_MAX_KEYS];
 
 /*
  * Write itup vector to page, has no control of free space.
@@ -144,12 +148,12 @@ gistfillitupvec(IndexTuple *vec, int veclen, int *memlen)
 }
 
 /*
- * Make unions of keys in IndexTuple vector (one union datum per index column).
- * Union Datums are returned into the attr/isnull arrays.
+ * Make unions of keys in IndexTuple vector.
  * Resulting Datums aren't compressed.
  */
+
 void
-gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len,
+gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len, int startkey,
 				   Datum *attr, bool *isnull)
 {
 	int			i;
@@ -158,12 +162,19 @@ gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len,
 
 	evec = (GistEntryVector *) palloc((len + 2) * sizeof(GISTENTRY) + GEVHDRSZ);
 
-	for (i = 0; i < giststate->tupdesc->natts; i++)
+	for (i = startkey; i < giststate->tupdesc->natts; i++)
 	{
 		int			j;
 
-		/* Collect non-null datums for this column */
 		evec->n = 0;
+		if (!isnull[i])
+		{
+			gistentryinit(evec->vector[evec->n], attr[i],
+						  NULL, NULL, (OffsetNumber) 0,
+						  FALSE);
+			evec->n++;
+		}
+
 		for (j = 0; j < len; j++)
 		{
 			Datum		datum;
@@ -181,7 +192,7 @@ gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len,
 			evec->n++;
 		}
 
-		/* If this column was all NULLs, the union is NULL */
+		/* If this tuple vector was all NULLs, the union is NULL */
 		if (evec->n == 0)
 		{
 			attr[i] = (Datum) 0;
@@ -191,7 +202,6 @@ gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len,
 		{
 			if (evec->n == 1)
 			{
-				/* unionFn may expect at least two inputs */
 				evec->n = 2;
 				evec->vector[1] = evec->vector[0];
 			}
@@ -214,12 +224,11 @@ gistMakeUnionItVec(GISTSTATE *giststate, IndexTuple *itvec, int len,
 IndexTuple
 gistunion(Relation r, IndexTuple *itvec, int len, GISTSTATE *giststate)
 {
-	Datum		attr[INDEX_MAX_KEYS];
-	bool		isnull[INDEX_MAX_KEYS];
+	memset(isnullS, TRUE, sizeof(bool) * giststate->tupdesc->natts);
 
-	gistMakeUnionItVec(giststate, itvec, len, attr, isnull);
+	gistMakeUnionItVec(giststate, itvec, len, 0, attrS, isnullS);
 
-	return gistFormTuple(giststate, r, attr, isnull, false);
+	return gistFormTuple(giststate, r, attrS, isnullS, false);
 }
 
 /*
@@ -231,15 +240,12 @@ gistMakeUnionKey(GISTSTATE *giststate, int attno,
 				 GISTENTRY *entry2, bool isnull2,
 				 Datum *dst, bool *dstisnull)
 {
-	/* we need a GistEntryVector with room for exactly 2 elements */
-	union
-	{
-		GistEntryVector gev;
-		char		padding[2 * sizeof(GISTENTRY) + GEVHDRSZ];
-	}			storage;
-	GistEntryVector *evec = &storage.gev;
+
 	int			dstsize;
 
+	static char storage[2 * sizeof(GISTENTRY) + GEVHDRSZ];
+	GistEntryVector *evec = (GistEntryVector *) storage;
+
 	evec->n = 2;
 
 	if (isnull1 && isnull2)
@@ -315,8 +321,6 @@ gistgetadjusted(Relation r, IndexTuple oldtup, IndexTuple addtup, GISTSTATE *gis
 				addentries[INDEX_MAX_KEYS];
 	bool		oldisnull[INDEX_MAX_KEYS],
 				addisnull[INDEX_MAX_KEYS];
-	Datum		attr[INDEX_MAX_KEYS];
-	bool		isnull[INDEX_MAX_KEYS];
 	IndexTuple	newtup = NULL;
 	int			i;
 
@@ -331,20 +335,19 @@ gistgetadjusted(Relation r, IndexTuple oldtup, IndexTuple addtup, GISTSTATE *gis
 		gistMakeUnionKey(giststate, i,
 						 oldentries + i, oldisnull[i],
 						 addentries + i, addisnull[i],
-						 attr + i, isnull + i);
+						 attrS + i, isnullS + i);
 
 		if (neednew)
 			/* we already need new key, so we can skip check */
 			continue;
 
-		if (isnull[i])
+		if (isnullS[i])
 			/* union of key may be NULL if and only if both keys are NULL */
 			continue;
 
 		if (!addisnull[i])
 		{
-			if (oldisnull[i] ||
-				!gistKeyIsEQ(giststate, i, oldentries[i].key, attr[i]))
+			if (oldisnull[i] || gistKeyIsEQ(giststate, i, oldentries[i].key, attrS[i]) == false)
 				neednew = true;
 		}
 	}
@@ -352,7 +355,7 @@ gistgetadjusted(Relation r, IndexTuple oldtup, IndexTuple addtup, GISTSTATE *gis
 	if (neednew)
 	{
 		/* need to update key */
-		newtup = gistFormTuple(giststate, r, attr, isnull, false);
+		newtup = gistFormTuple(giststate, r, attrS, isnullS, false);
 		newtup->t_tid = oldtup->t_tid;
 	}
 
@@ -432,7 +435,7 @@ gistchoose(Relation r, Page p, IndexTuple it,	/* it has compressed entry */
 			{
 				/*
 				 * New best penalty for column.  Tentatively select this tuple
-				 * as the target, and record the best penalty.  Then reset the
+				 * as the target, and record the best penalty.	Then reset the
 				 * next column's penalty to "unknown" (and indirectly, the
 				 * same for all the ones to its right).  This will force us to
 				 * adopt this tuple's penalty values as the best for all the
@@ -448,7 +451,7 @@ gistchoose(Relation r, Page p, IndexTuple it,	/* it has compressed entry */
 			{
 				/*
 				 * The current tuple is exactly as good for this column as the
-				 * best tuple seen so far.  The next iteration of this loop
+				 * best tuple seen so far.	The next iteration of this loop
 				 * will compare the next column.
 				 */
 			}
@@ -584,8 +587,10 @@ gistpenalty(GISTSTATE *giststate, int attno,
 	else if (isNullOrig && isNullAdd)
 		penalty = 0.0;
 	else
-		penalty = 1e10;			/* try to prevent mixing null and non-null
-								 * values */
+	{
+		/* try to prevent mixing null and non-null values */
+		penalty = get_float4_infinity();
+	}
 
 	return penalty;
 }
@@ -623,7 +628,7 @@ gistcheckpage(Relation rel, Buffer buf)
 	/*
 	 * ReadBuffer verifies that every newly-read page passes
 	 * PageHeaderIsValid, which means it either contains a reasonably sane
-	 * page header or is all-zero.  We have to defend against the all-zero
+	 * page header or is all-zero.	We have to defend against the all-zero
 	 * case, however.
 	 */
 	if (PageIsNew(page))
@@ -713,13 +718,30 @@ gistoptions(PG_FUNCTION_ARGS)
 {
 	Datum		reloptions = PG_GETARG_DATUM(0);
 	bool		validate = PG_GETARG_BOOL(1);
-	bytea	   *result;
+	relopt_value *options;
+	GiSTOptions *rdopts;
+	int			numoptions;
+	static const relopt_parse_elt tab[] = {
+		{"fillfactor", RELOPT_TYPE_INT, offsetof(GiSTOptions, fillfactor)},
+		{"buffering", RELOPT_TYPE_STRING, offsetof(GiSTOptions, bufferingModeOffset)}
+	};
 
-	result = default_reloptions(reloptions, validate, RELOPT_KIND_GIST);
+	options = parseRelOptions(reloptions, validate, RELOPT_KIND_GIST,
+							  &numoptions);
+
+	/* if none set, we're done */
+	if (numoptions == 0)
+		PG_RETURN_NULL();
+
+	rdopts = allocateReloptStruct(sizeof(GiSTOptions), options, numoptions);
+
+	fillRelOptions((void *) rdopts, sizeof(GiSTOptions), options, numoptions,
+				   validate, tab, lengthof(tab));
+
+	pfree(options);
+
+	PG_RETURN_BYTEA_P(rdopts);
 
-	if (result)
-		PG_RETURN_BYTEA_P(result);
-	PG_RETURN_NULL();
 }
 
 /*
