@@ -3,7 +3,7 @@
  * basebackup.c
  *	  code for taking a base backup and streaming it to a standby
  *
- * Portions Copyright (c) 2010-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2010-2012, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/replication/basebackup.c
@@ -18,15 +18,14 @@
 #include <time.h>
 
 #include "access/xlog_internal.h"		/* for pg_start/stop_backup */
-#include "catalog/catalog.h"
 #include "catalog/pg_type.h"
 #include "lib/stringinfo.h"
 #include "libpq/libpq.h"
 #include "libpq/pqformat.h"
-#include "miscadmin.h"
 #include "nodes/pg_list.h"
 #include "replication/basebackup.h"
 #include "replication/walsender.h"
+#include "replication/walsender_private.h"
 #include "storage/fd.h"
 #include "storage/ipc.h"
 #include "utils/builtins.h"
@@ -44,10 +43,9 @@ typedef struct
 } basebackup_options;
 
 
-static int64 sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces);
-static int64 sendTablespace(char *path, bool sizeonly);
-static bool sendFile(char *readfilename, char *tarfilename,
-		 struct stat * statbuf, bool missing_ok);
+static int64 sendDir(char *path, int basepathlen, bool sizeonly);
+static void sendFile(char *readfilename, char *tarfilename,
+		 struct stat * statbuf);
 static void sendFileWithContent(const char *filename, const char *content);
 static void _tarWriteHeader(const char *filename, const char *linktarget,
 				struct stat * statbuf);
@@ -69,7 +67,6 @@ typedef struct
 {
 	char	   *oid;
 	char	   *path;
-	char	   *rpath;			/* relative path within PGDATA, or NULL */
 	int64		size;
 } tablespaceinfo;
 
@@ -96,11 +93,9 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 	XLogRecPtr	startptr;
 	XLogRecPtr	endptr;
 	char	   *labelfile;
-	int			datadirpathlen;
-
-	datadirpathlen = strlen(DataDir);
 
 	startptr = do_pg_start_backup(opt->label, opt->fastcheckpoint, &labelfile);
+	SendXlogRecPtrResult(startptr);
 
 	PG_ENSURE_ERROR_CLEANUP(base_backup_cleanup, (Datum) 0);
 	{
@@ -109,14 +104,11 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 		struct dirent *de;
 		tablespaceinfo *ti;
 
-		SendXlogRecPtrResult(startptr);
-
 		/* Collect information about all tablespaces */
 		while ((de = ReadDir(tblspcdir, "pg_tblspc")) != NULL)
 		{
 			char		fullpath[MAXPGPATH];
 			char		linkpath[MAXPGPATH];
-			char	   *relpath = NULL;
 			int			rllen;
 
 			/* Skip special stuff */
@@ -143,23 +135,13 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 			}
 			linkpath[rllen] = '\0';
 
-			/*
-			 * Relpath holds the relative path of the tablespace directory
-			 * when it's located within PGDATA, or NULL if it's located
-			 * elsewhere.
-			 */
-			if (rllen > datadirpathlen &&
-				strncmp(linkpath, DataDir, datadirpathlen) == 0 &&
-				IS_DIR_SEP(linkpath[datadirpathlen]))
-				relpath = linkpath + datadirpathlen + 1;
-
 			ti = palloc(sizeof(tablespaceinfo));
 			ti->oid = pstrdup(de->d_name);
 			ti->path = pstrdup(linkpath);
-			ti->rpath = relpath ? pstrdup(relpath) : NULL;
-			ti->size = opt->progress ? sendTablespace(fullpath, true) : -1;
+			ti->size = opt->progress ? sendDir(linkpath, strlen(linkpath), true) : -1;
 			tablespaces = lappend(tablespaces, ti);
 #else
+
 			/*
 			 * If the platform does not have symbolic links, it should not be
 			 * possible to have tablespaces - clearly somebody else created
@@ -167,13 +149,13 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 			 */
 			ereport(WARNING,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-					 errmsg("tablespaces are not supported on this platform")));
+				  errmsg("tablespaces are not supported on this platform")));
 #endif
 		}
 
 		/* Add a node for the base directory at the end */
 		ti = palloc0(sizeof(tablespaceinfo));
-		ti->size = opt->progress ? sendDir(".", 1, true, tablespaces) : -1;
+		ti->size = opt->progress ? sendDir(".", 1, true) : -1;
 		tablespaces = lappend(tablespaces, ti);
 
 		/* Send tablespace header */
@@ -191,26 +173,29 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 			pq_sendint(&buf, 0, 2);		/* natts */
 			pq_endmessage(&buf);
 
+			/* In the main tar, include the backup_label first. */
+			if (ti->path == NULL)
+				sendFileWithContent(BACKUP_LABEL_FILE, labelfile);
+
+			sendDir(ti->path == NULL ? "." : ti->path,
+					ti->path == NULL ? 1 : strlen(ti->path),
+					false);
+
+			/* In the main tar, include pg_control last. */
 			if (ti->path == NULL)
 			{
 				struct stat statbuf;
 
-				/* In the main tar, include the backup_label first... */
-				sendFileWithContent(BACKUP_LABEL_FILE, labelfile);
-
-				/* ... then the bulk of the files ... */
-				sendDir(".", 1, false, tablespaces);
-
-				/* ... and pg_control after everything else. */
 				if (lstat(XLOG_CONTROL_FILE, &statbuf) != 0)
+				{
 					ereport(ERROR,
 							(errcode_for_file_access(),
 							 errmsg("could not stat control file \"%s\": %m",
 									XLOG_CONTROL_FILE)));
-				sendFile(XLOG_CONTROL_FILE, XLOG_CONTROL_FILE, &statbuf, false);
+				}
+
+				sendFile(XLOG_CONTROL_FILE, XLOG_CONTROL_FILE, &statbuf);
 			}
-			else
-				sendTablespace(ti->path, false);
 
 			/*
 			 * If we're including WAL, and this is the main data directory we
@@ -257,9 +242,7 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 		while (true)
 		{
 			/* Send another xlog segment */
-			char		sn[MAXPGPATH];
 			char		fn[MAXPGPATH];
-			char		pathbuf[MAXPGPATH];
 			int			i;
 
 			XLogFilePath(fn, ThisTimeLineID, logid, logseg);
@@ -292,15 +275,6 @@ perform_base_backup(basebackup_options *opt, DIR *tblspcdir)
 			 * boundary, so padding is never necessary.
 			 */
 
-			/*
-			 * Mark file as archived, otherwise files can get archived again
-			 * after promotion of a new node. This is in line with
-			 * walreceiver.c always doing a XLogArchiveForceDone() after a
-			 * complete segment.
-			 */
-			XLogFileName(sn, ThisTimeLineID, logid, logseg);
-			StatusFilePath(pathbuf, sn, ".done");
-			sendFileWithContent(pathbuf, "");
 
 			/* Advance to the next WAL file */
 			NextLogSeg(logid, logseg);
@@ -428,7 +402,7 @@ SendBaseBackup(BaseBackupCmd *cmd)
 	dir = AllocateDir("pg_tblspc");
 	if (!dir)
 		ereport(ERROR,
-				(errmsg("could not open directory \"pg_tblspc\": %m")));
+				(errmsg("could not open directory \"%s\": %m", "pg_tblspc")));
 
 	perform_base_backup(&opt, dir);
 
@@ -596,61 +570,13 @@ sendFileWithContent(const char *filename, const char *content)
 	}
 }
 
-/*
- * Include the tablespace directory pointed to by 'path' in the output tar
- * stream.  If 'sizeonly' is true, we just calculate a total length and return
- * it, without actually sending anything.
- *
- * Only used to send auxiliary tablespaces, not PGDATA.
- */
-static int64
-sendTablespace(char *path, bool sizeonly)
-{
-	int64		size;
-	char		pathbuf[MAXPGPATH];
-	struct stat statbuf;
-
-	/*
-	 * 'path' points to the tablespace location, but we only want to include
-	 * the version directory in it that belongs to us.
-	 */
-	snprintf(pathbuf, sizeof(pathbuf), "%s/%s", path,
-			 TABLESPACE_VERSION_DIRECTORY);
-
-	/*
-	 * Store a directory entry in the tar file so we get the permissions right.
-	 */
-	if (lstat(pathbuf, &statbuf) != 0)
-	{
-		if (errno != ENOENT)
-			ereport(ERROR,
-					(errcode_for_file_access(),
-					 errmsg("could not stat file or directory \"%s\": %m",
-							pathbuf)));
-
-		/* If the tablespace went away while scanning, it's no error. */
-		return 0;
-	}
-	if (!sizeonly)
-		_tarWriteHeader(TABLESPACE_VERSION_DIRECTORY, NULL, &statbuf);
-	size = 512;		/* Size of the header just added */
-
-	/* Send all the files in the tablespace version directory */
-	size += sendDir(pathbuf, strlen(path), sizeonly, NIL);
-
-	return size;
-}
-
 /*
  * Include all files from the given directory in the output tar stream. If
  * 'sizeonly' is true, we just calculate a total length and return it, without
  * actually sending anything.
- *
- * Omit any directory in the tablespaces list, to avoid backing up
- * tablespaces twice when they were created inside PGDATA.
  */
 static int64
-sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
+sendDir(char *path, int basepathlen, bool sizeonly)
 {
 	DIR		   *dir;
 	struct dirent *de;
@@ -695,6 +621,10 @@ sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
 			strcmp(pathbuf, "./postmaster.opts") == 0)
 			continue;
 
+		/* Skip pg_control here to back up it last */
+		if (strcmp(pathbuf, "./global/pg_control") == 0)
+			continue;
+
 		if (lstat(pathbuf, &statbuf) != 0)
 		{
 			if (errno != ENOENT)
@@ -726,24 +656,15 @@ sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
 				_tarWriteHeader(pathbuf + basepathlen + 1, NULL, &statbuf);
 			}
 			size += 512;		/* Size of the header just added */
-
-			/*
-			 * Also send archive_status directory (by hackishly reusing
-			 * statbuf from above ...).
-			 */
-			if (!sizeonly)
-				_tarWriteHeader("./pg_xlog/archive_status", NULL, &statbuf);
-			size += 512;		/* Size of the header just added */
-
 			continue;			/* don't recurse into pg_xlog */
 		}
 
 		/* Allow symbolic links in pg_tblspc only */
 		if (strcmp(path, "./pg_tblspc") == 0 &&
 #ifndef WIN32
-				 S_ISLNK(statbuf.st_mode)
+			S_ISLNK(statbuf.st_mode)
 #else
-				 pgwin32_is_junction(pathbuf)
+			pgwin32_is_junction(pathbuf)
 #endif
 			)
 		{
@@ -767,6 +688,7 @@ sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
 				_tarWriteHeader(pathbuf + basepathlen + 1, linkpath, &statbuf);
 			size += 512;		/* Size of the header just added */
 #else
+
 			/*
 			 * If the platform does not have symbolic links, it should not be
 			 * possible to have tablespaces - clearly somebody else created
@@ -774,15 +696,12 @@ sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
 			 */
 			ereport(WARNING,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-					 errmsg("tablespaces are not supported on this platform")));
+				  errmsg("tablespaces are not supported on this platform")));
 			continue;
-#endif /* HAVE_READLINK */
+#endif   /* HAVE_READLINK */
 		}
 		else if (S_ISDIR(statbuf.st_mode))
 		{
-			bool		skip_this_dir = false;
-			ListCell   *lc;
-
 			/*
 			 * Store a directory entry in the tar file so we can get the
 			 * permissions right.
@@ -791,44 +710,16 @@ sendDir(char *path, int basepathlen, bool sizeonly, List *tablespaces)
 				_tarWriteHeader(pathbuf + basepathlen + 1, NULL, &statbuf);
 			size += 512;		/* Size of the header just added */
 
-			/*
-			 * Call ourselves recursively for a directory, unless it happens
-			 * to be a separate tablespace located within PGDATA.
-			 */
-			foreach(lc, tablespaces)
-			{
-				tablespaceinfo *ti = (tablespaceinfo *) lfirst(lc);
-
-				/*
-				 * ti->rpath is the tablespace relative path within PGDATA, or
-				 * NULL if the tablespace has been properly located somewhere
-				 * else.
-				 *
-				 * Skip past the leading "./" in pathbuf when comparing.
-				 */
-				if (ti->rpath && strcmp(ti->rpath, pathbuf + 2) == 0)
-				{
-					skip_this_dir = true;
-					break;
-				}
-			}
-			if (!skip_this_dir)
-				size += sendDir(pathbuf, basepathlen, sizeonly, tablespaces);
+			/* call ourselves recursively for a directory */
+			size += sendDir(pathbuf, basepathlen, sizeonly);
 		}
 		else if (S_ISREG(statbuf.st_mode))
 		{
-			bool sent = false;
-
+			/* Add size, rounded up to 512byte block */
+			size += ((statbuf.st_size + 511) & ~511);
 			if (!sizeonly)
-				sent = sendFile(pathbuf, pathbuf + basepathlen + 1, &statbuf,
-								true);
-
-			if (sent || sizeonly)
-			{
-				/* Add size, rounded up to 512byte block */
-				size += ((statbuf.st_size + 511) & ~511);
-				size += 512;		/* Size of the header of the file */
-			}
+				sendFile(pathbuf, pathbuf + basepathlen + 1, &statbuf);
+			size += 512;		/* Size of the header of the file */
 		}
 		else
 			ereport(WARNING,
@@ -888,17 +779,9 @@ _tarChecksum(char *header)
 	return sum;
 }
 
-/*
- * Given the member, write the TAR header & send the file.
- *
- * If 'missing_ok' is true, will not throw an error if the file is not found.
- *
- * Returns true if the file was successfully sent, false if 'missing_ok',
- * and the file did not exist.
- */
-static bool
-sendFile(char *readfilename, char *tarfilename, struct stat *statbuf,
-		 bool missing_ok)
+/* Given the member, write the TAR header & send the file */
+static void
+sendFile(char *readfilename, char *tarfilename, struct stat * statbuf)
 {
 	FILE	   *fp;
 	char		buf[TAR_SEND_SIZE];
@@ -908,13 +791,9 @@ sendFile(char *readfilename, char *tarfilename, struct stat *statbuf,
 
 	fp = AllocateFile(readfilename, "rb");
 	if (fp == NULL)
-	{
-		if (errno == ENOENT && missing_ok)
-			return false;
 		ereport(ERROR,
 				(errcode_for_file_access(),
 				 errmsg("could not open file \"%s\": %m", readfilename)));
-	}
 
 	/*
 	 * Some compilers will throw a warning knowing this test can never be true
@@ -968,8 +847,6 @@ sendFile(char *readfilename, char *tarfilename, struct stat *statbuf,
 	}
 
 	FreeFile(fp);
-
-	return true;
 }
 
 
@@ -988,7 +865,7 @@ _tarWriteHeader(const char *filename, const char *linktarget,
 	memset(h, 0, sizeof(h));
 
 	/* Name 100 */
-	strlcpy(&h[0], filename, 100);
+	sprintf(&h[0], "%.99s", filename);
 	if (linktarget != NULL || S_ISDIR(statbuf->st_mode))
 	{
 		/*
@@ -1030,7 +907,7 @@ _tarWriteHeader(const char *filename, const char *linktarget,
 		/* Type - Symbolic link */
 		sprintf(&h[156], "2");
 		/* Link Name 100 */
-		strlcpy(&h[157], linktarget, 100);
+		sprintf(&h[157], "%.99s", linktarget);
 	}
 	else if (S_ISDIR(statbuf->st_mode))
 		/* Type - directory */
@@ -1047,11 +924,11 @@ _tarWriteHeader(const char *filename, const char *linktarget,
 
 	/* User 32 */
 	/* XXX: Do we need to care about setting correct username? */
-	strlcpy(&h[265], "postgres", 32);
+	sprintf(&h[265], "%.31s", "postgres");
 
 	/* Group 32 */
 	/* XXX: Do we need to care about setting correct group name? */
-	strlcpy(&h[297], "postgres", 32);
+	sprintf(&h[297], "%.31s", "postgres");
 
 	/* Major Dev 8 */
 	sprintf(&h[329], "%07o ", 0);
