@@ -5,13 +5,13 @@
  *
  *
  * For machines that have test-and-set (TAS) instructions, s_lock.h/.c
- * define the spinlock implementation.  This file contains only a stub
+ * define the spinlock implementation.	This file contains only a stub
  * implementation for spinlocks using PGSemaphores.  Unless semaphores
  * are implemented in a way that doesn't involve a kernel call, this
  * is too slow to be very useful :-(
  *
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -23,7 +23,6 @@
 #include "postgres.h"
 
 #include "miscadmin.h"
-#include "replication/walsender.h"
 #include "storage/lwlock.h"
 #include "storage/spin.h"
 
@@ -51,21 +50,14 @@ SpinlockSemas(void)
 int
 SpinlockSemas(void)
 {
-	int		nsemas;
-
 	/*
 	 * It would be cleaner to distribute this logic into the affected modules,
 	 * similar to the way shmem space estimation is handled.
 	 *
-	 * For now, though, there are few enough users of spinlocks that we just
-	 * keep the knowledge here.
+	 * For now, though, we just need a few spinlocks (10 should be plenty)
+	 * plus one for each LWLock and one for each buffer header.
 	 */
-	nsemas = NumLWLocks();		/* one for each lwlock */
-	nsemas += NBuffers;			/* one for each buffer header */
-	nsemas += max_wal_senders;	/* one for each wal sender process */
-	nsemas += 30;				/* plus a bunch for other small-scale use */
-
-	return nsemas;
+	return NumLWLocks() + NBuffers + 10;
 }
 
 /*
