@@ -3,7 +3,7 @@
  * ts_utils.c
  *		various support functions
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
@@ -18,15 +18,13 @@
 
 #include "miscadmin.h"
 #include "tsearch/ts_locale.h"
-#include "tsearch/ts_public.h"
 #include "tsearch/ts_utils.h"
-#include "utils/builtins.h"
 
 
 /*
  * Given the base name and extension of a tsearch config file, return
- * its full path name.  The base name is assumed to be user-supplied,
- * and is checked to prevent pathname attacks.  The extension is assumed
+ * its full path name.	The base name is assumed to be user-supplied,
+ * and is checked to prevent pathname attacks.	The extension is assumed
  * to be safe.
  *
  * The result is a palloc'd string.
@@ -39,7 +37,7 @@ get_tsearch_config_filename(const char *basename,
 	char	   *result;
 
 	/*
-	 * We limit the basename to contain a-z, 0-9, and underscores.  This may
+	 * We limit the basename to contain a-z, 0-9, and underscores.	This may
 	 * be overly restrictive, but we don't want to allow access to anything
 	 * outside the tsearch_data directory, so for instance '/' *must* be
 	 * rejected, and on some platforms '\' and ':' are risky as well. Allowing
@@ -64,12 +62,12 @@ get_tsearch_config_filename(const char *basename,
 static int
 comparestr(const void *a, const void *b)
 {
-	return strcmp(*(char **) a, *(char **) b);
+	return strcmp(*(char *const *) a, *(char *const *) b);
 }
 
 /*
  * Reads a stop-word file. Each word is run through 'wordop'
- * function, if given.  wordop may either modify the input in-place,
+ * function, if given.	wordop may either modify the input in-place,
  * or palloc a new version.
  */
 void
