@@ -3,7 +3,7 @@
  * enum.c
  *	  I/O functions, operators, aggregates etc for enum types
  *
- * Copyright (c) 2006-2011, PostgreSQL Global Development Group
+ * Copyright (c) 2006-2012, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
@@ -17,8 +17,6 @@
 #include "access/heapam.h"
 #include "catalog/indexing.h"
 #include "catalog/pg_enum.h"
-#include "catalog/pg_type.h"
-#include "fmgr.h"
 #include "libpq/pqformat.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
@@ -105,10 +103,6 @@ enum_recv(PG_FUNCTION_ARGS)
 	char	   *name;
 	int			nbytes;
 
-	/* guard against pre-9.3 misdeclaration of enum_recv */
-	if (get_fn_expr_argtype(fcinfo->flinfo, 0) == CSTRINGOID)
-		elog(ERROR, "invalid argument for enum_recv");
-
 	name = pq_getmsgtext(buf, buf->len - buf->cursor, &nbytes);
 
 	/* must check length to prevent Assert failure within SearchSysCache */
