@@ -3,7 +3,7 @@
  * float.c
  *	  Functions for the built-in floating-point types.
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -23,6 +23,7 @@
 #include "libpq/pqformat.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
+#include "utils/sortsupport.h"
 
 
 #ifndef M_PI
@@ -175,7 +176,11 @@ is_infinite(double val)
 
 
 /*
- *		float4in		- converts "num" to float4
+ *		float4in		- converts "num" to float
+ *						  restricted syntax:
+ *						  {<sp>} [+|-] {digit} [.{digit}] [<exp>]
+ *						  where <sp> is a space, digit is 0-9,
+ *						  <exp> is "e" or "E" followed by an integer.
  */
 Datum
 float4in(PG_FUNCTION_ARGS)
@@ -192,10 +197,6 @@ float4in(PG_FUNCTION_ARGS)
 	 */
 	orig_num = num;
 
-	/* skip leading whitespace */
-	while (*num != '\0' && isspace((unsigned char) *num))
-		num++;
-
 	/*
 	 * Check for an empty-string input to begin with, to avoid the vagaries of
 	 * strtod() on different platforms.
@@ -206,6 +207,10 @@ float4in(PG_FUNCTION_ARGS)
 				 errmsg("invalid input syntax for type real: \"%s\"",
 						orig_num)));
 
+	/* skip leading whitespace */
+	while (*num != '\0' && isspace((unsigned char) *num))
+		num++;
+
 	errno = 0;
 	val = strtod(num, &endptr);
 
@@ -215,14 +220,9 @@ float4in(PG_FUNCTION_ARGS)
 		int			save_errno = errno;
 
 		/*
-		 * C99 requires that strtod() accept NaN, [+-]Infinity, and [+-]Inf,
-		 * but not all platforms support all of these (and some accept them
-		 * but set ERANGE anyway...)  Therefore, we check for these inputs
-		 * ourselves if strtod() fails.
-		 *
-		 * Note: C99 also requires hexadecimal input as well as some extended
-		 * forms of NaN, but we consider these forms unportable and don't try
-		 * to support them.  You can use 'em if your strtod() takes 'em.
+		 * C99 requires that strtod() accept NaN and [-]Infinity, but not all
+		 * platforms support that yet (and some accept them but set ERANGE
+		 * anyway...)  Therefore, we check for these inputs ourselves.
 		 */
 		if (pg_strncasecmp(num, "NaN", 3) == 0)
 		{
@@ -234,36 +234,26 @@ float4in(PG_FUNCTION_ARGS)
 			val = get_float4_infinity();
 			endptr = num + 8;
 		}
-		else if (pg_strncasecmp(num, "+Infinity", 9) == 0)
-		{
-			val = get_float4_infinity();
-			endptr = num + 9;
-		}
 		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
 		{
 			val = -get_float4_infinity();
 			endptr = num + 9;
 		}
-		else if (pg_strncasecmp(num, "inf", 3) == 0)
-		{
-			val = get_float4_infinity();
-			endptr = num + 3;
-		}
-		else if (pg_strncasecmp(num, "+inf", 4) == 0)
-		{
-			val = get_float4_infinity();
-			endptr = num + 4;
-		}
-		else if (pg_strncasecmp(num, "-inf", 4) == 0)
-		{
-			val = -get_float4_infinity();
-			endptr = num + 4;
-		}
 		else if (save_errno == ERANGE)
-			ereport(ERROR,
-					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
-					 errmsg("\"%s\" is out of range for type real",
-							orig_num)));
+		{
+			/*
+			 * Some platforms return ERANGE for denormalized numbers (those
+			 * that are not zero, but are too close to zero to have full
+			 * precision).	We'd prefer not to throw error for that, so try to
+			 * detect whether it's a "real" out-of-range condition by checking
+			 * to see if the result is zero or huge.
+			 */
+			if (val == 0.0 || val >= HUGE_VAL || val <= -HUGE_VAL)
+				ereport(ERROR,
+						(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+						 errmsg("\"%s\" is out of range for type real",
+								orig_num)));
+		}
 		else
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
@@ -287,7 +277,7 @@ float4in(PG_FUNCTION_ARGS)
 
 	/*
 	 * In some IRIX versions, strtod() recognizes only "inf", so if the input
-	 * is "infinity" we have to skip over "inity".  Also, it may return
+	 * is "infinity" we have to skip over "inity".	Also, it may return
 	 * positive infinity for "-inf".
 	 */
 	if (isinf(val))
@@ -297,11 +287,6 @@ float4in(PG_FUNCTION_ARGS)
 			val = get_float4_infinity();
 			endptr = num + 8;
 		}
-		else if (pg_strncasecmp(num, "+Infinity", 9) == 0)
-		{
-			val = get_float4_infinity();
-			endptr = num + 9;
-		}
 		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
 		{
 			val = -get_float4_infinity();
@@ -397,6 +382,10 @@ float4send(PG_FUNCTION_ARGS)
 
 /*
  *		float8in		- converts "num" to float8
+ *						  restricted syntax:
+ *						  {<sp>} [+|-] {digit} [.{digit}] [<exp>]
+ *						  where <sp> is a space, digit is 0-9,
+ *						  <exp> is "e" or "E" followed by an integer.
  */
 Datum
 float8in(PG_FUNCTION_ARGS)
@@ -413,10 +402,6 @@ float8in(PG_FUNCTION_ARGS)
 	 */
 	orig_num = num;
 
-	/* skip leading whitespace */
-	while (*num != '\0' && isspace((unsigned char) *num))
-		num++;
-
 	/*
 	 * Check for an empty-string input to begin with, to avoid the vagaries of
 	 * strtod() on different platforms.
@@ -427,6 +412,10 @@ float8in(PG_FUNCTION_ARGS)
 			 errmsg("invalid input syntax for type double precision: \"%s\"",
 					orig_num)));
 
+	/* skip leading whitespace */
+	while (*num != '\0' && isspace((unsigned char) *num))
+		num++;
+
 	errno = 0;
 	val = strtod(num, &endptr);
 
@@ -436,14 +425,9 @@ float8in(PG_FUNCTION_ARGS)
 		int			save_errno = errno;
 
 		/*
-		 * C99 requires that strtod() accept NaN, [+-]Infinity, and [+-]Inf,
-		 * but not all platforms support all of these (and some accept them
-		 * but set ERANGE anyway...)  Therefore, we check for these inputs
-		 * ourselves if strtod() fails.
-		 *
-		 * Note: C99 also requires hexadecimal input as well as some extended
-		 * forms of NaN, but we consider these forms unportable and don't try
-		 * to support them.  You can use 'em if your strtod() takes 'em.
+		 * C99 requires that strtod() accept NaN and [-]Infinity, but not all
+		 * platforms support that yet (and some accept them but set ERANGE
+		 * anyway...)  Therefore, we check for these inputs ourselves.
 		 */
 		if (pg_strncasecmp(num, "NaN", 3) == 0)
 		{
@@ -455,36 +439,26 @@ float8in(PG_FUNCTION_ARGS)
 			val = get_float8_infinity();
 			endptr = num + 8;
 		}
-		else if (pg_strncasecmp(num, "+Infinity", 9) == 0)
-		{
-			val = get_float8_infinity();
-			endptr = num + 9;
-		}
 		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
 		{
 			val = -get_float8_infinity();
 			endptr = num + 9;
 		}
-		else if (pg_strncasecmp(num, "inf", 3) == 0)
-		{
-			val = get_float8_infinity();
-			endptr = num + 3;
-		}
-		else if (pg_strncasecmp(num, "+inf", 4) == 0)
-		{
-			val = get_float8_infinity();
-			endptr = num + 4;
-		}
-		else if (pg_strncasecmp(num, "-inf", 4) == 0)
-		{
-			val = -get_float8_infinity();
-			endptr = num + 4;
-		}
 		else if (save_errno == ERANGE)
-			ereport(ERROR,
-					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+		{
+			/*
+			 * Some platforms return ERANGE for denormalized numbers (those
+			 * that are not zero, but are too close to zero to have full
+			 * precision).	We'd prefer not to throw error for that, so try to
+			 * detect whether it's a "real" out-of-range condition by checking
+			 * to see if the result is zero or huge.
+			 */
+			if (val == 0.0 || val >= HUGE_VAL || val <= -HUGE_VAL)
+				ereport(ERROR,
+						(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
 				   errmsg("\"%s\" is out of range for type double precision",
 						  orig_num)));
+		}
 		else
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
@@ -508,7 +482,7 @@ float8in(PG_FUNCTION_ARGS)
 
 	/*
 	 * In some IRIX versions, strtod() recognizes only "inf", so if the input
-	 * is "infinity" we have to skip over "inity".  Also, it may return
+	 * is "infinity" we have to skip over "inity".	Also, it may return
 	 * positive infinity for "-inf".
 	 */
 	if (isinf(val))
@@ -518,11 +492,6 @@ float8in(PG_FUNCTION_ARGS)
 			val = get_float8_infinity();
 			endptr = num + 8;
 		}
-		else if (pg_strncasecmp(num, "+Infinity", 9) == 0)
-		{
-			val = get_float8_infinity();
-			endptr = num + 9;
-		}
 		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
 		{
 			val = -get_float8_infinity();
@@ -992,6 +961,24 @@ btfloat4cmp(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32(float4_cmp_internal(arg1, arg2));
 }
 
+static int
+btfloat4fastcmp(Datum x, Datum y, SortSupport ssup)
+{
+	float4		arg1 = DatumGetFloat4(x);
+	float4		arg2 = DatumGetFloat4(y);
+
+	return float4_cmp_internal(arg1, arg2);
+}
+
+Datum
+btfloat4sortsupport(PG_FUNCTION_ARGS)
+{
+	SortSupport ssup = (SortSupport) PG_GETARG_POINTER(0);
+
+	ssup->comparator = btfloat4fastcmp;
+	PG_RETURN_VOID();
+}
+
 /*
  *		float8{eq,ne,lt,le,gt,ge}		- float8/float8 comparison operations
  */
@@ -1088,6 +1075,24 @@ btfloat8cmp(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32(float8_cmp_internal(arg1, arg2));
 }
 
+static int
+btfloat8fastcmp(Datum x, Datum y, SortSupport ssup)
+{
+	float8		arg1 = DatumGetFloat8(x);
+	float8		arg2 = DatumGetFloat8(y);
+
+	return float8_cmp_internal(arg1, arg2);
+}
+
+Datum
+btfloat8sortsupport(PG_FUNCTION_ARGS)
+{
+	SortSupport ssup = (SortSupport) PG_GETARG_POINTER(0);
+
+	ssup->comparator = btfloat8fastcmp;
+	PG_RETURN_VOID();
+}
+
 Datum
 btfloat48cmp(PG_FUNCTION_ARGS)
 {
@@ -2050,7 +2055,7 @@ float8_stddev_samp(PG_FUNCTION_ARGS)
  * in that order.  Note that Y is the first argument to the aggregates!
  *
  * It might seem attractive to optimize this by having multiple accumulator
- * functions that only calculate the sums actually needed.  But on most
+ * functions that only calculate the sums actually needed.	But on most
  * modern machines, a couple of extra floating-point multiplies will be
  * insignificant compared to the other per-tuple overhead, so I've chosen
  * to minimize code space instead.
@@ -2745,7 +2750,7 @@ width_bucket_float8(PG_FUNCTION_ARGS)
 	if (isnan(operand) || isnan(bound1) || isnan(bound2))
 		ereport(ERROR,
 				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_WIDTH_BUCKET_FUNCTION),
-			  errmsg("operand, lower bound and upper bound cannot be NaN")));
+			 errmsg("operand, lower bound, and upper bound cannot be NaN")));
 
 	/* Note that we allow "operand" to be infinite */
 	if (isinf(bound1) || isinf(bound2))
