@@ -42,7 +42,6 @@ typedef double fsec_t;
 
 
 #define DAGO			"ago"
-#define DCURRENT		"current"
 #define EPOCH			"epoch"
 #define INVALID			"invalid"
 #define EARLY			"-infinity"
@@ -69,6 +68,7 @@ typedef double fsec_t;
 #define DA_D			"ad"
 #define DB_C			"bc"
 #define DTIMEZONE		"timezone"
+#define DCURRENT		   "current"
 
 /*
  * Fundamental time field definitions for parsing.
@@ -85,7 +85,7 @@ typedef double fsec_t;
 #define BC		1
 
 /*
- * Field types for time decoding.
+ * Fields for time decoding.
  *
  * Can't have more of these than there are bits in an unsigned int
  * since these are turned into bit masks during parsing and decoding.
@@ -103,9 +103,9 @@ typedef double fsec_t;
 #define YEAR	2
 #define DAY		3
 #define JULIAN	4
-#define TZ		5				/* fixed-offset timezone abbreviation */
-#define DTZ		6				/* fixed-offset timezone abbrev, DST */
-#define DYNTZ	7				/* dynamic timezone abbr (unimplemented) */
+#define TZ		5
+#define DTZ		6
+#define DTZMOD	7
 #define IGNORE_DTF	8
 #define AMPM	9
 #define HOUR	10
@@ -124,25 +124,19 @@ typedef double fsec_t;
 /* generic fields to help with parsing */
 #define ISODATE 22
 #define ISOTIME 23
-/* hack for parsing two-word timezone specs "MET DST" etc */
-#define DTZMOD	28				/* "DST" as a separate word */
 /* reserved for unrecognized string values */
 #define UNKNOWN_FIELD	31
 
 
 /*
  * Token field definitions for time parsing and decoding.
- *
- * Some field type codes (see above) use these as the "value" in datetktbl[].
- * These are also used for bit masks in DecodeDateTime and friends
+ * These need to fit into the datetkn table type.
+ * At the moment, that means keep them within [-127,127].
+ * These are also used for bit masks in DecodeDateDelta()
  *	so actually restrict them to within [0,31] for now.
  * - thomas 97/06/19
- * Not all of these fields are used for masks in DecodeDateTime
+ * Not all of these fields are used for masks in DecodeDateDelta
  *	so allow some larger than 31. - thomas 1997-11-17
- *
- * Caution: there are undocumented assumptions in the code that most of these
- * values are not equal to IGNORE_DTF nor RESERV.  Be very careful when
- * renumbering values in either of these apparently-independent lists :-(
  */
 
 #define DTK_NUMBER		0
@@ -198,24 +192,23 @@ typedef double fsec_t;
 #define DTK_DATE_M		(DTK_M(YEAR) | DTK_M(MONTH) | DTK_M(DAY))
 #define DTK_TIME_M		(DTK_M(HOUR) | DTK_M(MINUTE) | DTK_M(SECOND))
 
-/*
- * Working buffer size for input and output of interval, timestamp, etc.
- * Inputs that need more working space will be rejected early.  Longer outputs
- * will overrun buffers, so this must suffice for all possible output.  As of
- * this writing, PGTYPESinterval_to_asc() needs the most space at ~90 bytes.
- */
-#define MAXDATELEN		128
-/* maximum possible number of fields in a date string */
-#define MAXDATEFIELDS	25
-/* only this many chars are stored in datetktbl */
-#define TOKMAXLEN		10
+#define MAXDATELEN		63		/* maximum possible length of an input date
+								 * string (not counting tr. null) */
+#define MAXDATEFIELDS	25		/* maximum possible number of fields in a date
+								 * string */
+#define TOKMAXLEN		10		/* only this many chars are stored in
+								 * datetktbl */
 
 /* keep this struct small; it gets used a lot */
 typedef struct
 {
-	char		token[TOKMAXLEN + 1];	/* always NUL-terminated */
-	char		type;			/* see field type codes above */
-	int32		value;			/* meaning depends on type */
+#if defined(_AIX)
+	char	   *token;
+#else
+	char		token[TOKMAXLEN];
+#endif   /* _AIX */
+	char		type;
+	char		value;			/* this may be unsigned, alas */
 } datetkn;
 
 
@@ -257,7 +250,7 @@ do { \
  *	DAYS_PER_MONTH is very imprecise.  The more accurate value is
  *	365.2425/12 = 30.436875, or '30 days 10:29:06'.  Right now we only
  *	return an integral number of days, but someday perhaps we should
- *	also return a 'time' value to be used as well.  ISO 8601 suggests
+ *	also return a 'time' value to be used as well.	ISO 8601 suggests
  *	30 days.
  */
 #define DAYS_PER_MONTH	30		/* assumes exactly 30 days per month */
@@ -341,12 +334,12 @@ do { \
 
 int			DecodeInterval(char **, int *, int, int *, struct tm *, fsec_t *);
 int			DecodeTime(char *, int *, struct tm *, fsec_t *);
-int			EncodeDateTime(struct tm *, fsec_t, int *, char **, int, char *, bool);
-int			EncodeInterval(struct tm *, fsec_t, int, char *);
+int			EncodeDateTime(struct tm * tm, fsec_t fsec, bool print_tz, int tz, const char *tzn, int style, char *str, bool EuroDates);
+int			EncodeInterval(struct tm * tm, fsec_t fsec, int style, char *str);
 int			tm2timestamp(struct tm *, fsec_t, int *, timestamp *);
 int			DecodeUnits(int field, char *lowtoken, int *val);
 bool		CheckDateTokenTables(void);
-int			EncodeDateOnly(struct tm *, int, char *, bool);
+int			EncodeDateOnly(struct tm * tm, int style, char *str, bool EuroDates);
 int			GetEpochTime(struct tm *);
 int			ParseDateTime(char *, char *, char **, int *, int *, char **);
 int			DecodeDateTime(char **, int *, int, int *, struct tm *, fsec_t *, bool);
